@@ -17,7 +17,7 @@ if acme_path not in sys.path:
 from acme import ParallelMap
 
 def f(x, y, z=3, w=np.zeros((3, 1)), **kwargs):
-    return (sum(x) + y) * z * w.max()
+    return (x + y) * z * w.max()
 
 import time
 
@@ -29,6 +29,8 @@ def g(x, y, z=3):
     time.sleep(300)
     return (sum(x) + y) * z * arr.max()
 
+def arr_test(x, y):
+    return x + y
 
 # Prepare code to be executed using, e.g., iPython's `%run` magic command
 if __name__ == "__main__":
@@ -36,10 +38,13 @@ if __name__ == "__main__":
     # Test stuff within here...
     # pass
 
-    pmap = ParallelMap(g, np.arange(100), 2)
-    pmap.daemon.estimate_memuse()
+    with ParallelMap(arr_test, [np.ones((20,)), 2 * np.ones((20,)), 3 * np.ones((20,))], 4, result_shape=(None, 20)) as pmap:
+        results = pmap.compute()
 
-    # with pmap as p:
-    #     p.compute()
+    # pmap = ParallelMap(g, np.arange(100), 2)
+    # pmap.daemon.estimate_memuse()
+
+    # # with pmap as p:
+    # #     p.compute()
 
 
